# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{InTaiXuanJingSymbols} (and fuzzy permutations)
# 
# Meaning: Block 'Tai Xuan Jing Symbols'
#
return <<'END';
1D300	1D35F	Tai Xuan Jing Symbols
END
