# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{TaiLe} (and fuzzy permutations)
# 
# Meaning: Script 'TAI_LE'
#
return <<'END';
1950	196D	TAI_LE
1970	1974	TAI_LE
END
